/*
 * DragAndDropPlugin.java - Drag-and-Drop plugin
 * Copyright (C) 2000 Greg Merrill
 * Other contributors: Denis Lambot
 *                     Carmine Lucarelli
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.io.*;
import java.util.Iterator;
import java.util.List;

import org.gjt.sp.jedit.Buffer;
import org.gjt.sp.jedit.View;
import org.gjt.sp.jedit.EBMessage;
import org.gjt.sp.jedit.EBPlugin;
import org.gjt.sp.jedit.EditAction;
import org.gjt.sp.jedit.EditPane;
import org.gjt.sp.jedit.EditBus;
import org.gjt.sp.jedit.jEdit;
import org.gjt.sp.jedit.msg.EditPaneUpdate;
import org.gjt.sp.util.Log;

public class DragAndDropPlugin extends EBPlugin
{

	public void handleMessage(EBMessage message)
	{
		if(message instanceof EditPaneUpdate)
		{
			EditPaneUpdate editPaneUpdate = (EditPaneUpdate)message;
			if(editPaneUpdate.getWhat() == EditPaneUpdate.CREATED)
			{
				EditPane editPane = editPaneUpdate.getEditPane();
				DropTarget dropTarget = new DropTarget(
						editPane.getTextArea().getPainter(),
						new DragAndDropPlugin.DropHandler(editPane)
						);
			}
			else if(editPaneUpdate.getWhat() == EditPaneUpdate.DESTROYED)
			{
				EditPane editPane = editPaneUpdate.getEditPane();
				editPane.getTextArea().getPainter().setDropTarget(null);
			}
		}
	}

	static class DropHandler implements DropTargetListener
	{
		protected EditPane editPane;

		DropHandler(EditPane editPane)
		{
			this.editPane = editPane;
		}
		public void drop(DropTargetDropEvent dtde)
		{
			dtde.acceptDrop(DnDConstants.ACTION_COPY);
			try
			{
				Transferable t = dtde.getTransferable();
				if(t.isDataFlavorSupported(DataFlavor.plainTextFlavor))
				{
					InputStream in = (InputStream)t.getTransferData(DataFlavor.plainTextFlavor);
					StringBuffer sb = new StringBuffer();
					int input;
					while((input = in.read()) != -1)
					{
						// accept only basic ASCII characters
						if(input > 0 && input < 128)
						{
							sb.append((char)input);
						}
					}
					editPane.getTextArea().setSelectedText(sb.toString());
				}
				else
				{
					if(t.isDataFlavorSupported(DataFlavor.javaFileListFlavor))
					{
						Object data = t.getTransferData(DataFlavor.javaFileListFlavor);
						Iterator iterator = ((List)data).iterator();
						while(iterator.hasNext())
						{
							File file = (File)iterator.next();
							Buffer tmpBuf = jEdit.openFile(editPane.getView(), file.getAbsolutePath());
						}
					}
				}
			}
			catch(Exception e)
			{
				Log.log(Log.ERROR, this,
						"Error in drop() metthod. Reason: " + e.getMessage()
						);
				dtde.dropComplete(false);
				return;
			}

			dtde.dropComplete(true);
		}


		public void dragEnter(DropTargetDragEvent dtde)
		{
			// check if there is at least one File Type in the list, or if drag is text
			DataFlavor[] flavors = dtde.getCurrentDataFlavors();
			for(int i = 0; i < flavors.length; i++)
			{
				System.out.println(flavors[i].toString());
				if(flavors[i].isFlavorJavaFileListType() || flavors[i].equals(DataFlavor.plainTextFlavor))
				{
					dtde.acceptDrag(DnDConstants.ACTION_COPY);
					return;
				}
			}
			dtde.rejectDrag();
		}

		public void dragOver(DropTargetDragEvent e){}

		public void dragExit(DropTargetEvent e){}

		public void dropActionChanged(DropTargetDragEvent e){}
	}

}



/*
 * ChangeLog:
 *
 * Revision 0.2.4  2001/04/23  Carmine Lucarelli
 *   CVS screwup, needed a new tag
 *
 * Revision 0.2.3  2001/04/10  Carmine Lucarelli
 *   Commented out code to activate last created buffer (caused bugs when editpane was split)
 *   Added ability to paste drag and drop text selections
 *
 */

